import Flutter
import UIKit
import AgoraRtcKit

public class SwiftAgoravoicePlugin: NSObject, FlutterPlugin, AgoraRtcEngineDelegate {

    var agoraKit: AgoraRtcEngineKit!
    var delegate:AgoraRtcEngineDelegate!
    
    public static func register(with registrar: FlutterPluginRegistrar) {
        let channel = FlutterMethodChannel(name: "agoravoice", binaryMessenger: registrar.messenger())
        let instance = SwiftAgoravoicePlugin()
        registrar.addMethodCallDelegate(instance, channel: channel)
    }
    
   

    
    public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        
        if (call.method == "getPlatformVersion") {
            result("iOS " + UIDevice.current.systemVersion)
            
        } else if call.method == "init" {
            let args = call.arguments as? Dictionary<String, String>
            let appId = args?["appId"] ?? ""
            initializeAgoraEngine(appId: appId)
            result(true)
            
        } else if call.method == "join" {
            let args = call.arguments as? Dictionary<String, String>
            let accessTokenUrl = args?["accessTokenUrl"] ?? ""
            let channel = args?["channel"] ?? ""
            guard let accessToken = fetchAccessToken(accessTokenUrl: accessTokenUrl) else {
                return
            }
            joinChannel(accessToken: accessToken, channel: channel)
            result(true)
            
        } else if call.method == "leave" {
            leaveChannel()
            result(true)
            
        } else if call.method == "mute" {
            let args = call.arguments as? Dictionary<String, Bool>
            let mute = args?["mute"] ?? false
            muteAudio(mute: mute)
            result(true)
            
        } else if call.method == "speaker" {
            let args = call.arguments as? Dictionary<String, Bool>
            let enable = args?["enable"] ?? false
            enableSpeakerPhone(enable: enable)
            result(true)
        }
    }
    
    public func rtcEngine(_ engine: AgoraRtcEngineKit, didJoinedOfUid uid: UInt, elapsed: Int) {
        print("user has joined")
    }
    
    public func rtcEngine(_ engine: AgoraRtcEngineKit, didLeaveChannelWith stats: AgoraChannelStats) {
        print("user has left")
        
    }
    func fetchAccessToken(accessTokenUrl: String) -> String? {
        
        guard let accessTokenURL = URL(string: accessTokenUrl) else {
            return nil
        }
        
        return try? String.init(contentsOf: accessTokenURL, encoding: .utf8)
    }
    
    func initializeAgoraEngine(appId: String) {
        agoraKit = AgoraRtcEngineKit.sharedEngine(withAppId: appId, delegate: nil)
        delegate = self
    }
    
    func joinChannel(accessToken: String, channel: String) {
        agoraKit.joinChannel(byToken: accessToken, channelId: channel, info:nil, uid:0) {[unowned self] (sid, uid, elapsed) -> Void in

            self.agoraKit.setEnableSpeakerphone(true)
            //UIApplication.shared.isIdleTimerDisabled = true
        }
    }
    
    func leaveChannel() {
        agoraKit.leaveChannel(nil)
        //UIApplication.shared.isIdleTimerDisabled = false
    }
    
    func muteAudio(mute: Bool) {
        agoraKit.muteLocalAudioStream(mute)
    }
    
    func enableSpeakerPhone(enable: Bool) {
        agoraKit.setEnableSpeakerphone(enable)
    }
}

